import bge

def doorSendMessage():
	
	""" This function runs through the door sensors, checks the side where it is activated ("door_direction" property in the sensors) and sends a message to the door hinge, that will play the animation based in the direction where the door was activated.
	The message will be "normal_open" if the player collides with the front sensor and press the "E" key, or the message will be "reverse_open" if the player collides with the back sensor and press the "E" key. """
	
	cont = bge.logic.getCurrentController()
	scene = bge.logic.getCurrentScene()
	own = cont.owner
		
	# Sensors
	E = own.sensors["E"]
	collision = own.sensors["collision"]
	
	# Properties
	door_direction = own["door_direction"] # To check which side the door was activated
	
	# Objects
	parent = own.parent # The door base
	
	### INITIALIZE ###
	
	# If "E" is pressed and the player is colliding with the front sensor
	if E.positive and collision.positive and door_direction == "normal":
		for obj in parent.children: # Sends the message to all the children of the door base, especially the hinge
			own.sendMessage("normal_open", "body", obj.name)
	
	# If "E" is pressed and the player is colliding with the back sensor
	if E.positive and collision.positive and door_direction == "reverse":
		for obj in parent.children: # Sends the message to all the children of the door base, especially the hinge
			own.sendMessage("reverse_open", "body", obj.name)
	
def doorOpenClose():
	
	""" This function checks the door states, plays the animations and sounds of the open and closing door based in the messages received by the doorSendMessage() function."""
	
	cont = bge.logic.getCurrentController()
	scene = bge.logic.getCurrentScene()
	own = cont.owner
	parent = own.parent
	
	# Sensors
	normal_open = own.sensors["normal_open"] # Message sensor
	reverse_open = own.sensors["reverse_open"] # Message sensor
	
	# Actuators
	anim_open_close = cont.actuators["anim_open_close"] # Action actuator
	snd_open = cont.actuators["snd_open"] # Sound actuator
	snd_close = cont.actuators["snd_close"] # Sound actuator
	
	# Properties
	timer = own["timer"] # Timer property for the automatic door closing
	door_state = own["door_state"] # String property showing the door state, changed through property actuators
	
	# Variables
	action_frame = own.getActionFrame() # The actual animation frame of the hinge
	is_playing = own.isPlayingAction() # If the animation of the hinge is playing
	
	### INITIALIZE ###
	
	### ANIMATION ###
	# Normal close
	if door_state == "closed" and action_frame == 15.0 and is_playing == False: # The door will close if your state is "closed" and the last open door animation frame is active
		anim_open_close.frameStart, anim_open_close.frameEnd = 16.0, 30.0
		cont.activate(anim_open_close)
	
	# Normal open
	if normal_open.positive and door_state == "closed" and is_playing == False: # The door will open if received the open message, the state is "closed" and any other hinge animation is playing
		anim_open_close.frameStart, anim_open_close.frameEnd = 1.0, 15.0
		cont.activate(anim_open_close)
	
	# Reverse close
	if door_state == "closed" and action_frame == 45.0 and is_playing == False: # The door will close if your state is "closed" and the last open door animation frame is active
		anim_open_close.frameStart, anim_open_close.frameEnd = 46.0, 59.0
		cont.activate(anim_open_close)
	
	# Reverse open
	if reverse_open.positive and door_state == "closed" and is_playing == False: # The door will open if received the open message, the state is "closed" and any other hinge animation is playing
		anim_open_close.frameStart, anim_open_close.frameEnd = 30.0, 45.0
		cont.activate(anim_open_close)
	
	### SOUND ###
	# Open
	if action_frame > 1.0 and action_frame < 2.0 or action_frame > 30.0 and action_frame < 31.0: # Checks if the hinge animation is at the exact point for the sound
		snd_open.startSound()
	
	# Close
	if action_frame > 25.0 and action_frame < 26.0 or action_frame > 55.0 and action_frame < 56.0: # Checks if the hinge animation is at the exact point for the sound
		snd_close.startSound()
	
	
	
